package cc.fascinated.bat.features;

import cc.fascinated.bat.common.Serializable;
import com.google.gson.Gson;
import lombok.AllArgsConstructor;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.NonNull;
import org.bson.Document;

import java.util.HashMap;
import java.util.Map;

/**
 * @author Fascinated (fascinated7)
 */
@NoArgsConstructor
public class FeatureProfile extends Serializable {
    /**
     * The feature states
     */
    private Map<String, FeatureState> featureStates;

    /**
     * Gets the feature states
     *
     * @return the feature states
     */
    public FeatureState getFeatureState(@NonNull Feature feature) {
        if (!feature.isCanBeDisabled()) {
            return FeatureProfile.FeatureState.ENABLED;
        }
        String featureName = feature.getName().toUpperCase();
        if (!this.featureStates.containsKey(featureName)) {
            this.featureStates.put(featureName, feature.getDefaultState());
        }
        return this.featureStates.get(featureName);
    }

    /**
     * Gets whether the feature is enabled
     *
     * @return the feature state
     */
    public boolean isFeatureEnabled(Feature feature) {
        return this.getFeatureState(feature) == FeatureState.ENABLED;
    }

    /**
     * Gets whether the feature is disabled
     *
     * @return the feature state
     */
    public boolean isFeatureDisabled(Feature feature) {
        return this.getFeatureState(feature) == FeatureState.DISABLED;
    }

    /**
     * Enables the feature
     *
     * @param feature the feature to enable
     */
    public void enableFeature(Feature feature) {
        this.setFeatureState(feature, FeatureState.ENABLED);
    }

    /**
     * Disables the feature
     *
     * @param feature the feature to disable
     */
    public void disableFeature(Feature feature) {
        this.setFeatureState(feature, FeatureState.DISABLED);
    }

    /**
     * Sets the feature state
     *
     * @param feature the feature to set the state for
     * @param state   the state to set
     */
    public void setFeatureState(Feature feature, FeatureState state) {
        this.featureStates.put(feature.getName().toUpperCase(), state);
    }

    @Override
    public void reset() {
        this.featureStates = null;
    }

    @Override
    public void load(Document document, Gson gson) {
        this.featureStates = new HashMap<>();
        for (String key : document.keySet()) {
            this.featureStates.put(key, FeatureState.valueOf(document.getString(key)));
        }
    }

    @Override
    public Document serialize(Gson gson) {
        Document document = new Document();
        for (String key : this.featureStates.keySet()) {
            document.put(key, this.featureStates.get(key).name());
        }
        return document;
    }

    @AllArgsConstructor @Getter
    public enum FeatureState {
        ENABLED(":white_check_mark:"),
        DISABLED(":x:");

        /**
         * The emoji for the feature state
         */
        private final String emoji;
    }
}
